# (c) Copyright 2002-2003 Nuxeo SARL <http://nuxeo.com>
# (c) Copyright 2002 Prfecture du Bas-Rhin, France
# (c) Copyright 2002 CIRB, Belgique
# Authors: Florent Guillaume <mailto:fg@nuxeo.com>
#          Julien Jalon <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: FileDocument.py,v 1.17 2003/04/01 21:53:58 fermigier Exp $

"""
  File Document
"""

__version__ = '$Revision: 1.17 $'[11:-2]

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo

from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

from Products.NuxDocument.CMFNuxDocument import CMFNuxDocument

from BaseDocument import BaseDocument, BaseDocument_adder


factory_type_information = (
    {'id': 'File Document',
     'description': 'A File Document is editable through the web.',
     'title': '_portal_type_File Document',
     'content_icon': 'document_icon.gif',
     'product': 'NuxCPSDocuments',
     'factory': 'addFileDocument',
     'meta_type': 'File Document',
     'immediate_view': 'filedocument_edit_form',
     'allow_discussion': 1,
     'filter_content_types': 0,
     'actions': ({'id': 'view',
                  'name': '_action_view_',
                  'action': 'filedocument_view',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'filedocument_edit_form',
                  'permissions': (ModifyPortalContent,)},
                 {'id': 'download',
                  'name': '_action_download_',
                  'action': 'index_html',
                  'permissions': (View,)},
                 {'id': 'create',
                  'name': '_action_create_',
                  'action': 'filedocument_create_form',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'isdocument',
                  'name': 'isdocument',
                  'action': 'isdocument',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'issearchabledocument',
                  'name': 'issearchabledocument',
                  'action': 'issearchabledocument',
                  'visible': 0,
                  'permissions': ()},
                 ),
     },
    )




class FileDocument(BaseDocument, CMFNuxDocument):
    """File Document"""
    meta_type = 'File Document'

    security = ClassSecurityInfo()

    manage_options = BaseDocument.manage_options
    _properties = BaseDocument._properties

    def __init__(self, id, file='', **kw):
        CMFNuxDocument.__init__(self, id)
        BaseDocument.__init__(self, id, **kw)
        CMFNuxDocument.edit(self, None, file)

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, file=None, file_change=None, **kw):
        """Edit the document"""
        if file_change in ('change', None):
            if self._isNotEmpty(file):
                CMFNuxDocument.edit(self, None, file)
        elif file_change == 'delete':
            CMFNuxDocument.edit(self, None, '')
        BaseDocument.edit(self, **kw)

    # Make sure to use CMFDocument methods
    def index_html(self, RESPONSE=None):
        """Call index_html from CMFNuxDocument"""
        return CMFNuxDocument.index_html(self, RESPONSE=RESPONSE)

    def SearchableText(self):
        """Call SearchableText from CMFNuxDocument"""
        return CMFNuxDocument.SearchableText(self)

    def getIcon(self, *args, **kw):
        """Call getIcon from CMFNuxDocument"""
        return CMFNuxDocument.getIcon(self, *args, **kw)
 
    icon = getIcon # For the ZMI


InitializeClass(FileDocument)


def addFileDocument(dispatcher, id, REQUEST=None, **kw):
    """Add a File Document"""
    ob = FileDocument(id, **kw)
    return BaseDocument_adder(dispatcher, id, ob, REQUEST=REQUEST)

