# Copyright (c) 2002-2003 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Copyright (c) 2002 CIRB, Belgique
# Authors: Florent Guillaume <mailto:fg@nuxeo.com>
#          Julien Jalon <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: MailingList.py,v 1.17 2003/04/01 21:53:58 fermigier Exp $

"""
  Mailing List
"""

from Globals import InitializeClass
from Acquisition import aq_parent
from AccessControl import ClassSecurityInfo

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

from BaseDocument import BaseDocument, BaseDocument_adder


factory_type_information = (
    {'id': 'Mailing List',
     'description': "Une Mailing-list permet des envois groups d'e-mail.",
     'title': "_portal_type_Mailing-list",
     'content_icon': 'mailinglist_icon.gif',
     'product': 'NuxCPSDocuments',
     'meta_type': 'Mailing List',
     'factory': 'addMailingList',
     'immediate_view': 'mailinglist_edit_form',
     'allow_discussion': 1,
     'subscription_allowed': 1,
     'anonymous_subscription': 0,
     'filter_content_types': 0,
     'actions': ({'id': 'view',
                  'name': '_action_view_',
                  'action': 'mailinglist_view',
                  'permissions': (View,)},
                 {'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'mailinglist_edit_form',
                  'permissions': (ModifyPortalContent,)},
                 {'id': 'subscribe',
                  'name': '_action_(un)subscribe_',
                  'action': 'mailinglist_subscribe_form',
                  'permissions': (View,)},
                 {'id': 'sendmail',
                  'name': '_action_send_email_',
                  'action': 'mailinglist_sendmail_form',
                  'permissions': (View,)},
                 {'id': 'create',
                  'name': '_action_create_',
                  'action': 'mailinglist_create_form',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'isfunctionalobject',
                  'name': 'isfunctionalobject',
                  'action': 'isfunctionalobject',
                  'visible': 0,
                  'permissions': ()},
                 ),
     },
    )


class MailingList(BaseDocument):
    """
    Mailing List
    """
    meta_type = 'Mailing List'

    security = ClassSecurityInfo()

    _properties = BaseDocument._properties + (
        {'id': 'members', 'type': 'lines', 'mode': 'w', 
         'label': 'Members'},
        {'id': 'emails', 'type': 'lines', 'mode': 'w', 
         'label': 'Other emails'},
        {'id': 'add_wg_managers', 'type': 'boolean', 'mode': 'w', 
         'label': 'Add workgroup managers'},
        {'id': 'add_wg_members', 'type': 'boolean', 'mode':'w', 
         'label': 'Add workgroup members'},
        {'id': 'add_wg_visitors', 'type': 'boolean', 'mode':'w', 
         'label': 'Add workgroup visitors'},
        {'id': 'subscription_allowed', 'type': 'boolean', 'mode': 'w', 
         'label': 'Subscription allowed'},
        {'id': 'anonymous_subscription', 'type': 'boolean', 'mode': 'w', 
         'label': 'Anonymous subscription'},
        )
    members = []
    emails = []
    add_wg_managers = 0
    add_wg_members = 0
    add_wg_visitors = 0
    subscription_allowed = 1
    anonymous_subscription = 0

    #
    # Content accessors
    #
    def __init__(self, id, **kw):
        self.subscription_allowed = kw.get("subscription_allowed", 0)
        self.anonymous_subscription = kw.get("anonymous_subscription", 0)
        BaseDocument.__init__(self, id, **kw)

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, **kw):
        """Edit the document."""
        self.subscription_allowed = kw.get("subscription_allowed", 0)
        self.anonymous_subscription = kw.get("anonymous_subscription", 0)
        self.manage_changeProperties(**kw)
        self.reindexObject()

    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        """Used by the catalog for basic full text indexing."""
        return '%s %s' % (self.title, self.description)

    #
    # Behavior
    #
    security.declarePrivate('get_user_folder')
    def get_user_folder(self):
        return self.acl_users

    security.declarePrivate('get_member_email')
    def get_member_email(self, id):
        dtool = getToolByName(self, 'portal_metadirectories')
        entry = dtool.members.getEntry(id)
        if not entry:
            return ''
        email = entry.get('email')
        if not email: # XXX
            email = 'no_mail_for_%s@example.com' % id
        return email

    security.declarePrivate('get_group_emails')
    def get_group_emails(self, gid):
        # XXX should use portal_directory somehow
        aclu = self.get_user_folder()
        if not hasattr(aclu, 'getGroupById'):
            return []
        group = aclu.getGroupById(gid, None)
        if group is None:
            return []
        users = group.getUsers()
        emails = []
        for id in users:
            email = self.get_member_email(id)
            if email:
                emails.append(email)
        return emails

    security.declareProtected(View, 'get_emails')
    def get_emails(self, context=None):
        """Get the emails for this list

        If context is not None, use it instead of self
        as the place to lookup workgroup managers/members/visitors.
        """
        ids = []
        # wg managers, members or visitors
        if context is None:
            context = aq_parent(self)
        
        getContextUsers =  getattr(context, 'getContextUsers', None)

        if getContextUsers is not None:
            context_users = getContextUsers()
        else:
            context_users = None
            mlr = self.mergedLocalRoles(context)

        for var, role in (('add_wg_managers', 'WorkgroupManager'),
                          ('add_wg_members', 'WorkgroupMember'),
                          ('add_wg_visitors', 'WorkgroupVisitor')):
            if not getattr(self, var):
                continue
            if context_users is None:
                for id, roles in mlr.items():
                    if role in roles:
                        ids.append(id)
            else:
                for id in context_users.get(role, []):
                    ids.append(id)

        # members
        ids.extend(self.members)
        # treat ids
        emails = []
        for id in ids:
            if id.startswith('group:'):
                gid = id[len('group:'):]
                emails.extend(self.get_group_emails(gid))
            else:
                email = self.get_member_email(id)
                if email:
                    emails.append(email)
        # add explicit emails
        emails.extend(self.emails)

        # remove redundant or invalid emails
        okemails = []
        for email in emails:
            email = email.strip()
            if email.find('@') <= 0:
                continue
            if email not in okemails:
                okemails.append(email)

        return okemails


    security.declareProtected(View, 'send_email')
    def send_email(self,
                   from_address='nobody@example.com',
                   reply_to=None,
                   subject='No subject',
                   body='No body',
                   context=None,
                   content_type='text/plain',
                   charset='ISO-8859-15',
                   ):
        """Send a mail to the list

        If context is not None, use it instead of self as the place to lookup
        workgroup managers/members/visitors.
        """
        if not hasattr(self, 'MailHost'):
            return 0
        mailhost = self.MailHost
        mto = self.get_emails(context=context)
        mto = [email for email in mto if not email.startswith('no_mail_for')]
        if reply_to is None:
            reply_to = from_address
        content = """\
From: %s
To: %s
Reply-To: %s
Subject: %s
Content-Type: %s; charset=%
Mime-Version: 1.0

%s"""
        content = content % (
            from_address, reply_to, ', '.join(mto), subject, content_type, charset, body
        )
        mailhost.send(content,
                      mto=mto,
                      mfrom=from_address,
                      subject=subject,
                      encode='8bit')


    # Modified from CMFCore.utils
    # XXX should be moved to NuxUserGroups
    security.declarePrivate('mergedLocalRoles')
    def mergedLocalRoles(self, object):
        """
        Return a merging of object and its ancestors' __ac_local_roles__
        and __ac_local_group_roles__
        """
        merged = {}
        object = getattr(object, 'aq_inner', object)
        while 1:
            if hasattr(object, '__ac_local_roles__'):
                dict = object.__ac_local_roles__ or {}
                if callable(dict): dict = dict()
                for k, v in dict.items():
                    if merged.has_key(k):
                        merged[k] = merged[k] + v
                    else:
                        merged[k] = v
            # groups
            if hasattr(object, '__ac_local_group_roles__'):
                dict = object.__ac_local_group_roles__ or {}
                if callable(dict): dict = dict()
                for k, v in dict.items():
                    k = 'group:'+k
                    if merged.has_key(k):
                        merged[k] = merged[k] + v
                    else:
                        merged[k] = v
            # end groups
            if hasattr(object, 'aq_parent'):
                object = object.aq_parent
                object = getattr(object, 'aq_inner', object)
                continue
            if hasattr(object, 'im_self'):
                object = object.im_self
                object = getattr(object, 'aq_inner', object)
                continue
            break
        return merged

    #
    # Subscription
    #
    security.declareProtected(View, 'subscribe')
    def getSubscriber(self, REQUEST=None):
        """Subscribe user to Post."""
        stool = getToolByName(self, "portal_subscription")
        return stool.getMemberOrMail(REQUEST)

    security.declareProtected(View, 'isSubscribed')
    def isSubscribed(self, REQUEST=None):
        """Return the user subscription state."""
        stool = getToolByName(self, "portal_subscription")
        return stool.isSubscribed(stool.getMemberOrMail(REQUEST), self)

    security.declareProtected(View, 'unsubscribe')
    def unsubscribe(self, REQUEST=None):
        """Remove user subscription."""
        stool = getToolByName(self, "portal_subscription")
        return stool.unsubscribe(stool.getMemberOrMail(REQUEST), self)

    security.declareProtected(View, 'subscribe')
    def subscribe(self, REQUEST=None):
        """Subscribe user to Post."""
        stool = getToolByName(self,"portal_subscription")
        return stool.subscribe(stool.getMemberOrMail(REQUEST), self)

    security.declareProtected(View, 'listSubscriptionsFor')
    def listSubscriptions(self, REQUEST=None):
        """Return observed objects for current user."""
        stool = getToolByName(self,"portal_subscription")
        return [ object for object in stool.listObservedObjectsFor(
                     stool.getMemberOrMail(REQUEST)) 
                 if object.portal_type == self.portal_type ]

InitializeClass(MailingList)

def addMailingList(dispatcher, id, REQUEST=None, **kw):
    """Add a Mailing List."""
    ob = MailingList(id, **kw)
    return BaseDocument_adder(dispatcher, id, ob, REQUEST=REQUEST)

