#
# (C) Copyright 2002 Nuxeo SARL <contact@nuxeo.com>
#
# This product is governed by a license. See LICENSE.txt file.
#

# Zope
import Globals
from zLOG import LOG, DEBUG, INFO
from Post import Post
from DateTime import DateTime
from interfaces.Forum import IForum
from AccessControl import ClassSecurityInfo

# Python
from types import *

# CMF
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import View, ManageProperties, \
    AddPortalContent, setDefaultRoles

# CPS
from Products.NuxCPSDocuments.cpsdocument_utils import getImageForFile
from Products.NuxWorkgroup.Workgroup import Workgroup, ManageWorkgroups
from Products.NuxCPSDocuments.BaseDocument import BaseDocument

# Forum Moderator Permissions
ManageForumContent = 'Manage Forum Content'
setDefaultRoles(ManageForumContent, ('Manager', 'WorkgroupManager'))


factory_type_information = (
  {'id': 'Forum',
   'meta_type': 'Forum',
   'description': "Forums hold threaded discussions",
   'icon': 'forum_icon.gif',
   'product': 'NuxCPSForum',
   'factory': 'addForum',
   'filter_content_types': 1,
   'allowed_content_types': ('Post',),
   'immediate_view': 'forum_edit_form',
   'actions': (
                {
                  'id': 'view',
                  'name': '_action_view_',
                  'action': 'forum_listing',
                  'permissions': (View,),
                }, {
                  'id': 'localroles',
                  'name': '_action_access_rights_',
                  'action': 'workgroup_localrole_form',
                  'permissions': (ManageWorkgroups,),
                  'category': 'object',
                }, {
                  'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'forum_edit_form',
                  'permissions': (ManageProperties,),
                }, {
                  'id': 'forum_post_message',
                  'name': '_action_forum_post_',
                  'action': 'forum_post_message',
                  'permissions': (AddPortalContent,),
                }, {
                  'id': 'isfunctionalobject',
                  'name': 'isfunctionalobject',
                  'action': 'isfunctionalobject',
                  'visible': 0,
                  'permissions': (),
                },
              )
 },
)

class Forum(Workgroup):

    meta_type = 'Forum'
    __implements__ = IForum

    security = ClassSecurityInfo()

    _properties = Workgroup._properties + (
        {'id': 'attachments_allowed', 'type': 'boolean', 'mode': 'w', 
         'label': 'Attachments allowed in posts'},
    )

    isDocumentContainer = 0
    isPrincipiaFolderish = 0

    # Default attributes
    isprivate = 0
    notification_on = 1
    attachments_allowed = 1
    isprio_moderation = 1
    ispost_moderation = 1
    subscription_allowed = 1
    sorting = 'date'
    msg_per_page = 15
    image = None
    
    def __init__(self, id, title=''):
        """Constructor."""
        Workgroup.__init__(self, id, title=title)

    def manage_afterAdd(self, item, container):
        """Correctly set Forum permissions."""
        Workgroup.manage_afterAdd(self, item, container)
        if item is self:
            self.manage_permission('Delete objects', 
                ['Manager', 'WorkgroupManager'], 0)
            self.manage_permission('Modify portal content', 
                ['Manager', 'WorkgroupManager', 'Owner'], 0)
    
    security.declareProtected(ManageProperties, 'edit')    
    def edit(self, title='', description=''):
        """Edition method from ancestor."""
        Workgroup.edit(self, title=title, description=description)
        
    security.declareProtected(ManageProperties, 'setForumProperties')    
    def setForumProperties(self, isprivate, attachments_allowed,
                           isprio_moderation, ispost_moderation,
                           subscription_allowed=0, sorting='date', 
                           msg_per_page=15, image=None,
                           image_change='keep'):
        """Set the Forum properties for the instance."""
        #
        # Forum preferences
        #
        self.isprivate = isprivate
        self.attachments_allowed = attachments_allowed
        self.isprio_moderation = isprio_moderation
        self.ispost_moderation = ispost_moderation
        # XXX: where is it used ? Fix this.
        self.subscription_allowed = subscription_allowed
        self.sorting = sorting
        self.msg_per_page = msg_per_page
        if image_change == 'change':
            if image is not None:
                self.image = getImageForFile(image=image)
        elif image_change == 'delete':
            self.image = None

    security.declareProtected(View, 'getUserName')    
    def getUserName(self):
        """Return the user's login name."""
        mtool = getToolByName(self, 'portal_membership')
        return mtool.getAuthenticatedMember().getMemberId()
    
    security.declareProtected(View, 'isModerator')    
    def isModerator(self):
        """Return wether the current user can manage this forum."""
        mtool = getToolByName(self, 'portal_membership')
        return mtool.checkPermission(ManageProperties, self)
        
    security.declareProtected(View, 'listPosts')    
    def listPosts(self):
        """Return the top-level posts."""
        mtool = getToolByName(self, 'portal_membership')
        posts = [ child for child in self.contentValues(['Post'])
                        if mtool.checkPermission(View, child) ]
        return posts

    children = listPosts

    security.declareProtected(View, 'getForum')    
    def getForum(self):
        """Return the Forum object."""
        return self

    security.declareProtected(View, 'getForumImage')
    def getForumImage(self):
        """Return the Forum image."""
        return self.image

    security.declareProtected(View, 'getMail')    
    def getMail(self, login):
        """Return the user mail reference to login."""
        # Use the CPS MetaDirectories
        members = getToolByName(self, 'portal_metadirectories').members
        t = members.getEntry(login)
        if t is not None:
            return t['email']
        else:
            return ""

    security.declareProtected(View, 'getMail')    
    def getMail(self, login):
        """Return the user mail reference to login."""
        # Use the CPS MetaDirectories
        members = getToolByName(self, 'portal_metadirectories').members
        t = members.getEntry(login)
        if t is not None:
            return t['email']
        else:
            return ""

    security.declareProtected(View, 'isSubscribed')    
    def isSubscribed(self):
        """Return the user subscription state."""
        mtool = getToolByName(self, 'portal_membership')
        ntool = getToolByName(self, 'portal_forum_notification')
        member = mtool.getAuthenticatedMember()
        return ntool.isSubscribed(member,self)

    security.declareProtected(View, 'unsubscribe')    
    def unsubscribe(self):
        """Remove user subscription to Post"""
        mtool = getToolByName(self, 'portal_membership')
        ntool = getToolByName(self, 'portal_forum_notification')
        member = mtool.getAuthenticatedMember()
        return ntool.unsubscribe(member,self)

    security.declareProtected(View, 'subscribe')    
    def subscribe(self):
        """Subscribe user to Post"""
        mtool = getToolByName(self, 'portal_membership')
        ntool = getToolByName(self, 'portal_forum_notification')
        member = mtool.getAuthenticatedMember()
        return ntool.subscribe(member,self)

    security.declareProtected(View, 'listObservedObjects')
    def listObservedObjects(self):
        """Return observed objects for current user."""
        mtool = getToolByName(self, 'portal_membership')
        ntool = getToolByName(self, 'portal_forum_notification')
        member = mtool.getAuthenticatedMember()
        return ntool.listObservedObjectsFor(member)

    #
    # Snapshots
    #
    security.declareProtected(AddPortalContent, 'createSnapShot')
    def createSnapShot(self, id, title, thread):
        """Create a thread snapshot in this Forum's container."""
        container = self.aq_inner.aq_parent
        container.invokeFactory('ThreadSnapShot', id=id, title=title,
                                original_thread=thread)
        return getattr(container, id)


def addForum(self, id, title='', REQUEST=None):
    """Add a Forum object."""
    forum = Forum(id, title)
    self._setObject(id, forum)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

Globals.InitializeClass(Forum)
