#
# (C) Copyright 2002 Nuxeo SARL <http://www.nuxeo.com/>
# This product is governed by a license. See LICENSE.txt file.
#

# Zope
from zLOG import LOG, DEBUG, INFO
from Acquisition import aq_base, aq_parent, aq_inner
from DateTime import DateTime
from AccessControl import ClassSecurityInfo
from AccessControl.User import nobody
import Globals, string

# CMF
from Products.CMFDefault.DublinCore      import DefaultDublinCoreImpl
from Products.CMFCore.utils              import getToolByName
from Products.CMFDefault.Document        import Document
from Products.NuxDocument.CMFNuxDocument import CMFNuxDocument

from Products.CMFCore.CMFCorePermissions import View, AddPortalContent,\
     ModifyPortalContent, ManageProperties

# CPS
from Products.NuxCPSDocuments.BaseDocument import BaseDocument

# This product
from interfaces.Post import IPost
from util import generateId

factory_type_information = (
    {
        'id': 'Post',
        'meta_type': 'Post',
        'description': "Messages in Forums",
        'icon': 'icon_post.gif',
        'product': 'NuxCPSForum',
        'factory': 'addPost',
        'filter_content_types': 1,
        'subscription_allowed': 1,
        'allowed_content_types': ('Post',),
        'immediate_view': 'metadata_edit_form',
        'global_allow': 0,
        'actions': (
            {
                'id': 'view',
                'name': '_action_view_',
                'action': 'post_view',
                'permissions': (View,),
            }, {
                'id': 'post_reply',
                'name': '_action_reply_',
                'action': 'forum_post_message',
                'permissions': (AddPortalContent,),
            }, {
                'id': 'edit',
                'name': '_action_modify_',
                'action': 'post_edit_form',
                'permissions': (ModifyPortalContent,)
            }, {
                'id': 'delete',
                'name': '_action_delete_',
                'action': 'post_delete',
                'permissions': ('Delete Post',)
            }, {
                'id': 'notification',
                'name': '_action_notification_',
                'action': 'forum_notification',
                'visible': 0,                
                'permissions': (View,)
            }, {
                'id': 'issearchabledocument',
                'name': 'issearchabledocument',
                'action': 'issearchabledocument',
                'visible': 0,
                'permissions': (),
            },
        )
    },
)

class Post(BaseDocument, Document):

    meta_type = 'Post'
    __implements__ = IPost
    
    _properties = BaseDocument._properties + (
        {'id': 'breadcrumb_skip', 'type': 'boolean', 
         'mode': 'w', 'label': 'Breadcrumb skip'},
    )

    breadcrumb_skip = 1
    edit_history = ()
    attachment = None
    security = ClassSecurityInfo()

    def __init__(self, id,
                 title='', description='', text='', author='', email='',
                 attachment=None):
        DefaultDublinCoreImpl.__init__(self)
        self.id = id
        self.title = title
        self.description = description
        self.author = author
        self.email = email
        self._edit(text=text, text_format='plain')
        self.setFormat('plain')
        self.created = DateTime()
        self.locked = 0 
        self.in_workflow = 0
        if attachment:
            self.attachment = CMFNuxDocument(id='attachment', file=attachment)

    security.declareProtected(ModifyPortalContent, "edit")
    def edit(self, title='', text='', attachment=None, attachment_change=None):
        """Edition method (protected)."""
        mtool = getToolByName(self, 'portal_membership')
        member = mtool.getAuthenticatedMember().getUserName()
        date = DateTime()
        self.edit_history += ({'member': member, 'date': date}, )

        self.title = title
        if self.attachments_allowed:
            self._editAttachment(attachment, attachment_change)
        self._edit(text=text, text_format='plain')
        
        self.reindexObject()

    def _editAttachment(self, attachment, attachment_change):
        if attachment_change in ('change', None):
            if attachment is not None and attachment.filename:
                self.attachment = CMFNuxDocument(id='attachment',
                                                 file=attachment)
            else:
                self.attachment = None
        elif attachment_change == 'delete':
            self.attachment = None

    security.declareProtected('Delete Post', 'delete')
    def delete(self, REQUEST=None):
        """Delete this post."""

        parent = self.aq_inner.aq_parent
        if parent is not None:
            parent.manage_delObjects([self.id])
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(parent.absolute_url())

    security.declareProtected(ManageProperties, "lockDiscussion")
    def lockDiscussion(self):
        """Lock a discussion."""
        self.locked = 1

        #roles_for_ForumManagers = self.rolesOfPermission(ManageProperties)
        #LOG('NCF', DEBUG, 'Roles: %s' % (roles_for_ForumManagers, ))
        #roles = []
        #for item in roles_for_ForumManagers:
        #    if item['selected'] == 'SELECTED':
        #        roles.append(item['name'])
        roles = ['WorkgroupManager', 'Manager']

        self.manage_permission(AddPortalContent, roles, 0)
        self.manage_permission(ModifyPortalContent, roles, 0)
        
    security.declareProtected(ManageProperties, "unlockDiscussion")
    def unlockDiscussion(self):
        """Unlock a discussion."""
        self.locked = 0

        self.manage_permission(AddPortalContent, [], 1)
        self.manage_permission(ModifyPortalContent, [], 1)
        
    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        """Return a string with all value properties.

        Used by the catalog for basic fulltext indexing.
        """
        if self.attachment is not None:
            more_text = self.attachment.PrincipiaSearchSource()
        else:
            more_text = ''
        return '%s %s %s %s' % (
            self.title, self.description, self.text, more_text)

    security.declareProtected(View, 'thread_top')
    def thread_top(self):
        """Return the top ancestor."""
        ancestors = self.ancestors()
        if len(ancestors) > 0:
            return ancestors[-1]
        else:
            return self

    security.declareProtected(View, 'thread_top')
    def getDescendants(self):
        """Return the descendants in this thread."""
        l = []
        for descendant in self.children():
            if descendant.meta_type == 'Post':
                l.append(descendant)
            l += descendant.getDescendants()

        # LOG('PostDebugLog', 0, "", "descendants[%s] = %s" % (self.id, str(l)))
        return l

    security.declareProtected(View, 'ancestors')
    def ancestors(self):
        """Return a list of ancestral posts."""
        ancestors = []
        parent = aq_parent(aq_inner(self))
        pm = getToolByName(self, 'portal_membership')
        while parent.meta_type == 'Post':
            if pm.checkPermission(View, parent):
                ancestors.append(parent)
            parent = aq_parent(aq_inner(parent))
        return ancestors

    security.declareProtected(View, 'children')
    def children(self, sort_key='id', REQUEST=None):
        """Return a list of replies to this item."""
        children = []
        pm = getToolByName(self, 'portal_membership')
        
        for child in self.contentValues(['Post']):
            LOG('NCF', DEBUG, 'Testing %s' % (child.title_or_id(), ))
            if pm.checkPermission(View, child):
                LOG('NCF', DEBUG, 'Ok.')
                children.append(child)
                
        children.sort(lambda x, y, sort_key=sort_key:
                      cmp(getattr(x, sort_key, None) ,
                          getattr(y, sort_key, None)))
        return children

    security.declareProtected(View, 'inReplyTo')
    def inReplyTo(self):
        """Return the object for which this object is a reply."""
        parent = self.aq_inner.aq_parent
        pm = getToolByName(self, 'portal_membership')
        if parent.meta_type == 'Post' and pm.checkPermission(View, parent):
            return parent
        else:
            return None

    security.declareProtected(View, 'nextReply')
    def nextReply(self):
        """Return the next object that is a reply."""
        pm = getToolByName(self, 'portal_membership')
        LOG('NCF', DEBUG, 'here: %s' % (self.title_or_id(), ))
        LOG('NCF', DEBUG, 'objects: %s' % (self.objectIds(), ))
        ids = self.contentIds(['Post'])
        LOG('NCF', DEBUG, 'ids: %s' % (ids, ))
        if ids:
            for id in ids:
                obj = getattr(self, id)
                if pm.checkPermission(View, obj):
                    return obj
        return None

    security.declareProtected(View, 'nextMessageInThread')
    def nextMessageInThread(self, replies=1):
        """Return the next object in this thread."""
        if replies:
            nextReply = self.nextReply()
            if nextReply:
                return nextReply
        nextMessage = self.nextMessage()
        if nextMessage:
            return nextMessage
        parent = self.aq_inner.aq_parent
        if hasattr(parent.aq_base, 'nextMessageInThread'):
            obj = parent.nextMessageInThread(replies=0)
            return obj
        return None

    security.declareProtected(View, 'nextMessage')
    def nextMessage(self):
        """Return the next object on the same level."""
        pm = getToolByName(self, 'portal_membership')
        if pm.checkPermission(View, self.aq_inner.aq_parent):
            ids = self.aq_inner.aq_parent.contentIds(['Post'])
            ids.sort()
            index = ids.index(self.getId())
            next = index + 1
            if next < len(ids):
                if pm.checkPermission(View, getattr(self, ids[next], None)):
                    return getattr(self, ids[next], None)
                else:
                    return None
        return None

    security.declareProtected(View, 'previousMessageInThread')
    def previousMessageInThread(self):
        """Return the previous object in this thread."""
        previousMessage = self.previousMessage()
        if previousMessage:
            return previousMessage
        parent = self.aq_inner.aq_parent
        if parent.meta_type == 'Post':
            return parent
        else:
            return None

    security.declareProtected(View, 'previousMessage')
    def previousMessage(self):
        """Return the previous object on the same level."""
        pm = getToolByName(self, 'portal_membership')
        if pm.checkPermission(View, self.aq_inner.aq_parent):
            ids = self.aq_inner.aq_parent.contentIds(['Post'])
            ids.sort()
            index = ids.index(self.getId())
            previous = index - 1
            if previous >= 0:
                if pm.checkPermission(View, getattr(self, ids[previous], None)):
                    return getattr(self, ids[previous], None)
                else:
                    return None
        return None

    security.declareProtected(View, 'childMessageCount')
    def childMessageCount(self):
        """Count the number of children of this message, including this message."""
        count=1 # this message
        for message in self.children():
            count = count + message.childMessageCount()
        return count

    security.declareProtected(View, 'threadMessageCount')
    def threadMessageCount(self):
        """Looks at the topmost ancestor, and then counts children."""
        message_count = self.thread_top().childMessageCount()
        return message_count

    security.declareProtected(View, 'childNewestMessageDate')
    def childNewestMessageDate(self):
        """Look through children and return the newest message date."""
        newest_date = self.created
        for message in self.children():
            test_date = message.childNewestMessageDate()
            if test_date > newest_date:
                newest_date = test_date
        return newest_date

    security.declareProtected(View, 'unsubscribe')    
    def unsubscribe(self):
        """Remove user subscription to Post"""
        mtool = getToolByName(self, 'portal_membership')
        ntool = getToolByName(self, 'portal_forum_notification')
        member = mtool.getAuthenticatedMember()
        if member != nobody:
            return ntool.isSubscribed(member,self)
        else:
            raise "Anonymous subscription NOT implemented yet"

    security.declareProtected(View, 'subscribe')    
    def subscribe(self):
        """Subscribe user to Post"""
        mtool = getToolByName(self, 'portal_membership')
        ntool = getToolByName(self, 'portal_forum_notification')
        member = mtool.getAuthenticatedMember()
        if member != nobody:
            return ntool.isSubscribed(member,self)
        else:
            raise "Anonymous subscription NOT implemented yet"

    security.declareProtected(View, 'isSubscribed')    
    def isSubscribed(self):
        """Return the user subscription state."""
        mtool = getToolByName(self, 'portal_membership')
        ntool = getToolByName(self, 'portal_forum_notification')
        member = mtool.getAuthenticatedMember()
        if member != nobody:
            return ntool.isSubscribed(member,self)
        else:
            raise "Anonymous subscription NOT implemented yet"

    security.declareProtected(View, 'threadNewestMessageDate')
    def threadNewestMessageDate(self):
        """Look at the topmost ancestor, and then recurse
        through the messages finding the last created message."""
        newest_date = self.thread_top().childNewestMessageDate()
        return newest_date

    def insertIntoWorkflow(self, REQUEST=None):
        if REQUEST is not None:
            # Can't be called TTW
            return

        if self.in_workflow:
            return

        self.in_workflow = 1
        forum = self.aq_inner.aq_parent
        moderate = forum.isprio_moderation
        if moderate:
            LOG('NCF', DEBUG, 'Moderating this post')
            self.setWorkflowSetChain(['post_workflow'])
            LOG('NCF', DEBUG, 'Notifying WF')
            self.notifyWorkflowCreated()
            LOG('NCF', DEBUG, 'Done')

    security.declareProtected(View, 'isSubscribed')    
    def isSubscribed(self):
        """Return the user subscription state."""
        mtool = getToolByName(self, 'portal_membership')
        ntool = getToolByName(self, 'portal_forum_notification')
        member = mtool.getAuthenticatedMember()
        return ntool.isSubscribed(member,self)

    security.declareProtected(View, 'unsubscribe')    
    def unsubscribe(self):
        """Remove user subscription to Post"""
        mtool = getToolByName(self, 'portal_membership')
        ntool = getToolByName(self, 'portal_forum_notification')
        member = mtool.getAuthenticatedMember()
        return ntool.unsubscribe(member,self)

    security.declareProtected(View, 'subscribe')    
    def subscribe(self):
        """Subscribe user to Post"""
        mtool = getToolByName(self, 'portal_membership')
        ntool = getToolByName(self, 'portal_forum_notification')
        member = mtool.getAuthenticatedMember()
        return ntool.subscribe(member,self)

def addPost(dispatcher, id='', title='', text='', author='', email='',
            attachment=None, REQUEST=None):
    """Add a Post to a container."""

    here = dispatcher.Destination()

    if not id:
        id = generateId(here)
    if (not here.attachments_allowed) \
      or (attachment and not attachment.filename):
        attachment = None

    post = Post(id, title=title, text=text, author=author, email=email,
                attachment=attachment)
    here._setObject(id, post)
    post = getattr(here, id)

    if post.notification_on:
        post.forum_notification_notify()

    if REQUEST is not None:
        return here.manage_main(self, REQUEST)

Globals.InitializeClass(Post)

