# (c) 2002 Nuxeo SARL <http://nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: PortalSubscriptionTool.py,v 1.4 2003/06/12 08:43:02 fermigier Exp $
"""
   PortalSubscriptionTool
"""

__version__='$Revision: 1.4 $'[11:-2]

from zLOG import LOG, INFO

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem

from Products.CMFCore.CMFCorePermissions import View
from Products.CMFCore.utils import UniqueObject, getToolByName

class PortalSubscriptionTool(UniqueObject, SimpleItem):
    """Portal Subscription Tool.
    
    A central (unique for each CPS) registry of subscriptions.
    """

    id = 'portal_subscription'
    meta_type = 'Portal Subscription Tool'
    security = ClassSecurityInfo()

    security.declareProtected('__init__')
    def __init__(self):
        """Creates an empty notification tool"""
        self._members = ()
        self._emails = ()

    def isSubscribed(self, someone, object):
        if not someone:
            someone = self.getMemberOrMail()

        if hasattr(someone, "getMemberId"):
            comp_list = self._members
            someone = someone.getMemberId()
        else:
            comp_list = self._emails

        object_path = "/".join(object.getPhysicalPath())
        for (some_id, link) in comp_list:
            if some_id == someone and link["path"] == object_path:
                return 1
        return 0

    security.declareProtected(View, 'subscribe')
    def subscribe(self, someone, object):
        """Subscribes either the member or the provided anonymous email to the
        service."""
        if not someone:
            someone = self.getMemberOrMail()

        if not self.isSubscribed(someone, object):
            object_link = { "path": "/".join(object.getPhysicalPath()),
                            "meta_type": object.meta_type}
            if hasattr(someone, "getMemberId"):
                self._members += ((someone.getMemberId(), object_link),)
            else:
                self._emails += ((someone, object_link),)

    security.declareProtected(View, 'unsubscribe')
    def unsubscribe(self, someone, object=None):
        """Unsubscribe the member to object, or remove all subscriptions 
        if object == None."""

        if not someone:
            someone = self.getMemberOrMail()
            
        if hasattr(someone, "getMemberId"):
            member_id = someone.getMemberId()
            if object == None:
                l = [ (user, link) for (user, link) 
                        in self._members if member_id != user ]
            else:
                l = []
                object_path = "/".join(object.getPhysicalPath())
                for (user, link) in self._members:
                    if member_id != user or link["path"] != object_path:
                        l.append((user, link))
            self._members = tuple(l)
        else:
            if object == None:
                l = [ (email, link) for (email, link) 
                        in self._emails if someone != email ]
            else:
                l = []
                object_path = "/".join(object.getPhysicalPath())
                for (email, link) in self._emails:
                    if someone != email or link["path"] != object_path:
                        l.append((email, link))
            self._emails = tuple(l)

    security.declareProtected(View, 'unsubscribeAll')
    def unsubscribeAll(self, member, paths):
        for path in paths:
            #log("unsubscribing path = "+path)
            object = self.unrestrictedTraverse(path)
            self.unsubscribe(member, object)

    security.declareProtected(View, 'listSubscribersFor')
    def listSubscribersFor(self, object):
        """Return list of member for object."""

        object_path = "/".join(object.getPhysicalPath())
        d = {}
        for (user, link) in self._members:
            if object_path.startswith(link["path"]):
                d[user] = None
        for (email, link) in self._emails:
            if object_path.startswith(link["path"]):
                d["email:" + str(email)] = None
        return [ user for user in d.keys() ]

    security.declareProtected(View, 'listObservedObjectsFor')
    def listObservedObjectsFor(self, someone):
        """Return list of objects observed by member."""

        if not someone:
            someone = self.getMemberOrMail()

        search_list = self._emails
        if hasattr(someone, "getMemberId"):
            someone = someone.getMemberId()
            search_list = self._members

        l = []
        for (something, link) in search_list:
            if something == someone:
                object = self.unrestrictedTraverse(link["path"], None)
                if object:
                    l.append(object)

        return l

    security.declareProtected(View, 'getMemberOrMail')
    def getMemberOrMail(self, REQUEST=None):
        """Returns any available authentication information. The return object
        is a Member, an email or fails. Failure result is None when a request
        is provided and NobodyException."""

        pmtool = getToolByName(self, "portal_membership")
        member = pmtool.getAuthenticatedMember()

        if member.getId() is not None:
            return member

        if REQUEST is not None:
            if hasattr(REQUEST,"cookies"):
                if REQUEST.cookies.has_key("email"):
                    return REQUEST.cookies["email"]

        return None

    def debug(self, message):
        LOG('SubscriptionTool', 0, '', message + '\n')


InitializeClass(PortalSubscriptionTool)
