# (c) 2002 Nuxeo SARL <http://nuxeo.com>
# (c) 2002 Florent Guillaume <mailto:fg@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: WorkflowAware.py,v 1.5 2002/09/05 13:04:33 jalon Exp $

"""
  WorkflowAware mixin.
"""

__version__='$Revision: 1.5 $'[11:-2]

import string

from zLOG import LOG, DEBUG

from Acquisition import aq_base
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass, DTMLFile
#from Products.PageTemplate.PageTemplateFile import PageTemplateFile
from DateTime.DateTime import DateTime

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import \
     setDefaultRoles, View, ModifyPortalContent
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl


ManageObjectsWorkflow = 'Manage Objects Workflows'
setDefaultRoles(ManageObjectsWorkflow, ('Manager',))


class WorkflowAware:
    """
    WorkflowAware Mixin.
    Give objects the capability to choose in which workflows it
    participates.
    """

    _isWorkflowAware = 1

    _workflow_object_setchain = None # Complete chain to use
    _workflow_object_addchain = () # Additional chain to use
    _workflow_object_ignorechain = () # Chain to ignore

    security = ClassSecurityInfo()

    #
    # Workflow Aware accessors
    #

    security.declareProtected(ManageObjectsWorkflow, 'setWorkflowSetChain')
    def setWorkflowSetChain(self, chain=None):
        """
        Set the complete workflow chain to use.
        """
        if chain is not None:
            chain = tuple(chain)
        self._workflow_object_setchain = chain

    security.declareProtected(ManageObjectsWorkflow, 'setWorkflowAddChain')

    def setWorkflowAddChain(self, chain=()):
        """
        Set the additional workflow chain to use.
        """
        chain = tuple(chain)
        self._workflow_object_addchain = chain

    security.declareProtected(ManageObjectsWorkflow, 'setWorkflowIgnoreChain')
    def setWorkflowIgnoreChain(self, chain=()):
        """
        Set the workflow chain to ignore.
        """
        chain = tuple(chain)
        self._workflow_object_ignorechain = chain

    security.declareProtected(ManageObjectsWorkflow, 'getWorkflowSetChain')
    def getWorkflowSetChain(self):
        """
        Get the complete workflow chain to use.
        """
        return self._workflow_object_setchain

    security.declareProtected(ManageObjectsWorkflow, 'getWorkflowAddChain')

    def getWorkflowAddChain(self):
        """
        Get the additional workflow chain to use.
        """
        return self._workflow_object_addchain

    security.declareProtected(ManageObjectsWorkflow, 'getWorkflowIgnoreChain')
    def getWorkflowIgnoreChain(self):
        """
        Get the workflow chain to ignore.
        """
        return self._workflow_object_ignorechain

    #
    # ZMI
    #

    security.declareProtected(ManageObjectsWorkflow, 'manage_workflowsTab')
    manage_workflowsTab = DTMLFile('www/zmi_workflows', globals())

    security.declareProtected(ManageObjectsWorkflow, 'manage_changeWorkflows')
    def manage_changeWorkflows(self, force, setchain, addchain, ignorechain,
                               REQUEST=None):
        """
        Update the Workflow Aware properties.
        """
        if force:
            setchain = string.replace(setchain, ',', ' ')
            setchain = filter(None, string.split(setchain, ' '))
        else:
            setchain = None
        addchain = string.replace(addchain, ',', ' ')
        ignorechain = string.replace(ignorechain, ',', ' ')
        addchain = filter(None, string.split(addchain, ' '))
        ignorechain = filter(None, string.split(ignorechain, ' '))
        self.setWorkflowSetChain(setchain)
        self.setWorkflowAddChain(addchain)
        self.setWorkflowIgnoreChain(ignorechain)
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(self.absolute_url()+'/manage_workflowsTab?manage_tabs_message=Changed.')

    #
    # Workflow Aware hooks
    #

    security.declarePrivate('workflowGetChain')
    def workflowGetChain(self, ob, chain, pwf=None):
        """
        Hook called by Workflow Tool after computing the object chain.
        Return the workflow chain to use.
        """
        setchain = self._workflow_object_setchain
        if setchain is not None:
            return setchain
        chain = list(chain)
        addchain = self._workflow_object_addchain
        ignorechain = self._workflow_object_ignorechain
        for id in addchain:
            if id not in chain:
                chain.append(id)
        for id in ignorechain:
            if id in chain:
                chain.remove(id)
        return tuple(chain)

    security.declarePrivate('workflowDoAction')
    def workflowDoAction(self, action, wf_id, args, kw, pwf=None):
        """
        Hook called by Workflow Tool before calling wf.doActionFor.
        Do an action, invoked by user interface.
        """
        pass

    #
    # Unused for now
    #

    security.declarePrivate('workflowNotifyBefore')
    def workflowNotifyBefore(self, action):
        pass

    security.declarePrivate('workflowNotifyException')
    def workflowNotifyException(self, action, exc):
        pass

    security.declarePrivate('workflowNotifyPrivate')
    def workflowNotifySuccess(self, action, res):
        pass


InitializeClass(WorkflowAware)
