# Copyright (c) 2002 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 CIRB, Belgique
# Author: Julien <mailto:jj@nuxeo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: BaseDirectory.py,v 1.30 2003/06/12 09:17:33 fermigier Exp $

"""
    Base Directory
    and
    DirectorySchemaProperty
    $Id: BaseDirectory.py,v 1.30 2003/06/12 09:17:33 fermigier Exp $
"""

# __version__ = '$Revision: 1.30 $'[11:-2]

from zLOG import LOG, INFO
from urllib import quote_plus
from OFS.Image import File
from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo, Unauthorized
from ZODB.PersistentMapping import PersistentMapping
import OFS
from Products.CMFCore.utils import UniqueObject, SimpleItemWithProperties,\
    getToolByName
from Products.CMFCore.CMFCorePermissions import setDefaultRoles
from Products.CMFCore.Expression import Expression
from Products.PageTemplates.Expressions import getEngine

from Acquisition import aq_parent, aq_inner, aq_base


class MalformedProperties(Exception):
    def __init__(self, malformed_properties):
        self.properties = malformed_properties

def malformed(title='_Malformed_', exception=MalformedProperties):
    raise exception, title

DirectoryEditPermission = 'Edit Directory'
setDefaultRoles(DirectoryEditPermission, ('Manager', ))

class DirectorySchemaProperty(SimpleItemWithProperties):
    """
        Class describing a directory schema property
    """

    meta_type = "Schema Property"

    security = ClassSecurityInfo()

    title = ''
    alias = ''
    directory_ref = '' # only usefull if isdirectory_ref
    isdirectory_ref = 0
    type = 'string'
    acl_read = None
    acl_write = None
    process_write = None
    process_read = None
    isexternal = 0
    issearchable = 1
    isvisible = 1
    nomenclature = None

    def __init__(self, id, alias=None, title=None, acl_read=None,
                 acl_write=None, process_read=None, process_write=None,
                 nomenclature=None, type=None, directory_ref=None,
                 isdirectory_ref=None, isexternal=None, isvisible=None,
                 issearchable=None):
        self.id = id
        if alias is not None:
            self.alias = alias
        if title is not None:
            self.title = title
        if directory_ref is not None:
            self.directory_ref = directory_ref
        if isdirectory_ref is not None:
            self.isdirectory_ref = isdirectory_ref
        if acl_read is not None:
            if acl_read:
                self.acl_read = Expression(acl_read)
            else:
                self.acl_read = None
        if acl_write is not None:
            if acl_write:
                self.acl_write = Expression(acl_write)
            else:
                self.acl_write = None
        if process_read is not None:
            if process_read:
                self.process_read = Expression(process_read)
            else:
                self.process_read = None
        if process_write is not None:
            if process_write:
                self.process_write = Expression(process_write)
            else:
                self.process_write = None
        if type is not None:
            self.type = type
        if isexternal is not None:
            self.isexternal = not not isexternal
        if isvisible is not None:
            self.isvisible = not not isvisible
        if issearchable is not None:
            self.issearchable = not not issearchable
        if nomenclature is not None:
            self.nomenclature = Expression(nomenclature)
       
    def _edit(self, id=None, alias=None, title=None, acl_read=None,
              acl_write=None, process_read=None, directory_ref=None,
              isdirectory_ref=None, process_write=None, nomenclature=None,
              type=None, isexternal=None, isvisible=None, issearchable=None):
        if id is not None:
            self.id = id
        if alias is not None:
            self.alias = alias
        if title is not None:
            self.title = title
        if directory_ref is not None:
            self.directory_ref = directory_ref
        if isdirectory_ref is not None:
            self.isdirectory_ref = isdirectory_ref
        if acl_read is not None:
            if acl_read:
                self.acl_read = Expression(acl_read)
            else:
                self.acl_read = None
        if acl_write is not None:
            if acl_write:
                self.acl_write = Expression(acl_write)
            else:
                self.acl_write = None
        if process_read is not None:
            if process_read:
                self.process_read = Expression(process_read)
            else:
                self.process_read = None
        if process_write is not None:
            if process_write:
                self.process_write = Expression(process_write)
            else:
                self.process_write = None
        if type is not None:
            self.type = type
        if isexternal is not None:
            self.isexternal = not not isexternal
        if isvisible is not None:
            self.isvisible = not not isvisible
        if issearchable is not None:
            self.issearchable = not not issearchable
        if nomenclature is not None:
            if nomenclature:
                self.nomenclature = Expression(nomenclature)
            else:
                self.nomenclature = None

    security.declarePublic('getNomenclature')
    def getNomenclature(self):
        if not self.nomenclature:
            return None
        directory = aq_parent(aq_inner(self))
        base_context = directory._getBaseExprContext()
        context = directory._createExprContext(base_context,
                            nomenclatures=directory.nomenclatures)
        nomenclature = self.nomenclature(context)
        if type(nomenclature) is not type({}):
            # nomenclature is a list so
            # change it to a dict
            if nomenclature is type(''):
                nomenclature = [nomenclature]
            nomen_dict = {}
            for key in nomenclature:
                nomen_dict[key] = key
            nomenclature = nomen_dict
        return nomenclature

    security.declarePrivate('getInternalName')
    def getInternalName(self):
        return self.alias or self.id
    
InitializeClass(DirectorySchemaProperty)

class BaseDirectory(OFS.Folder.Folder):
    """
        Base class for a Directory
    """

    meta_type = 'Base Directory'

    security = ClassSecurityInfo()

    manage_options = (SimpleItemWithProperties.manage_options[:1] +
                      (
                        {'label': 'Schema',
                         'action': 'manage_editSchemaForm',
                        },
                      ) +
                      SimpleItemWithProperties.manage_options[1:])

    _basic_properties = (
        {'id':'title', 'type': 'string', 'mode':'w',
            'label':'Title'},
        )

    _properties = _basic_properties

    acl_read = None
    acl_write = None
    acl_create = None
    acl_delete = None
    process_aftercreate = None
    process_beforedelete = None
    entry_prop = 'id'
    display_prop = 'id'

    # templates
    search_template = 'directory_searchform'
    results_template = 'directory_viewresults'
    entry_template = 'directory_getentry'
    edit_template = 'directory_editentry_form'


    def __init__(self, id, title=''):
        self.id = id
        self.title = title
        self._schema = PersistentMapping()
        self._schemakeys = ()
        self.nomenclatures = PersistentMapping()

    # using the directory

    security.declarePublic('readablePropertiesForEntry')
    def readablePropertiesForEntry(self, entry):
        """
            returns the list of readable properties based on acl_read
        """
        return self._priv_readablePropertiesForEntry(entry)

    def _priv_readablePropertiesForEntry(self, entry, base_context=None):
        readable_props = []
        for prop in self._schemakeys:
            property = self._schema[prop]
            if property.acl_read:
                if base_context is None:
                    base_context = self._getBaseExprContext()
                context = self._createExprContext(base_context,
                                                  entry=entry,
                                                  property=entry.get(prop,None))
                if property.acl_read(context):
                    readable_props.append(prop)
                else:
                    pass
            else:
                readable_props.append(prop)
        
        return readable_props

    security.declarePublic('writablePropertiesForEntry')
    def writablePropertiesForEntry(self, entry, entry_id):
        """
            returns the list of readable properties based on acl_read
        """
        return self._priv_writablePropertiesForEntry(entry, entry_id)

    def _priv_writablePropertiesForEntry(self, entry, entry_id,
                                         base_context=None):
        writable_props = []
        for prop in self._schema.keys():
            property = self._schema[prop]
            if property.acl_write:
                if base_context is None:
                    base_context = self._getBaseExprContext()
                context = self._createExprContext(base_context,
                                                  entry=entry,
                                                  entry_id=entry_id,
                                                  property=entry.get(prop, None))
                if property.acl_write(context):
                    writable_props.append(prop)
            else:
                writable_props.append(prop)
        
        return writable_props

    security.declarePublic('can_edit')
    def can_edit(self, entry):
        """
            Test if current member can edit the entry
        """
        return self._priv_can_edit(entry)

    def _priv_can_edit(self, entry, base_context=None):
        if not self.acl_write:
            return 1

        if base_context is None:
            base_context = self._getBaseExprContext()

        context = self._createExprContext(base_context,
                                          entry=entry,
                                          entry_id=entry[self.entry_prop])
        return not not self.acl_write(context)

    security.declarePublic('can_delete')
    def can_delete(self, entry):
        """
            Test if current member can delete an entry
        """
        return self._priv_can_delete(entry)

    def _priv_can_delete(self, entry, base_context=None):
        if not self.acl_delete:
            return 1

        if base_context is None:
            base_context = self._getBaseExprContext()

        context = self._createExprContext(base_context, entry=entry, entry_id=entry[self.entry_prop])
        return not not self.acl_delete(context)
    
    security.declarePublic('can_create')
    def can_create(self):
        """
            Test if current member can create an entry
        """
        return self._priv_can_create()

    def _priv_can_create(self, base_context=None):
        if not self.acl_create:
            return 1

        if base_context is None:
            base_context = self._getBaseExprContext()

        context = self._createExprContext(base_context)
        return not not self.acl_create(context)
    
    security.declarePublic('searchEntry')
    def searchEntry(self, search_options=None, **kw):
        """
            search for entries in the directory
        """
        return self._priv_searchEntry(search_options=search_options,
                                      base_context=None, **kw)

    def _priv_searchEntry(self, base_context=None, search_options=None, **kw):
        # first clean search arguments
        for key in kw.keys():
            value = kw[key]
            if type(value) is type(''):
                value = value.strip()
                kw[key] = value
            if not value:
                del kw[key]
        # separate internal searchargs and external searchargs
        internal_searchargs = {}
        external_searchargs = {}
        searchkeys = kw.keys()
        for prop in self._schemakeys:
            property = self._schema[prop]
            aliasprop = property.alias or prop
            if prop in searchkeys:
                if not property.isexternal:
                    internal_searchargs[aliasprop] = kw[prop]
                else:
                    # no aliasing for external properties
                    external_searchargs[prop] = kw[prop]

        if base_context is None:
            base_context = self._getBaseExprContext()

        entries = self._searchEntry(search_options, **internal_searchargs)

        returned_entries = []
        prop_list = self._schemakeys
        entry_alias = self._schema[self.entry_prop].getInternalName()
        for entry in entries:
            # first process entry
            processed_entry = self._process_read(entry,
                                    entry_id=entry[entry_alias],
                                    base_context=base_context)

            # test readability of the entry
            if self.acl_read:
                context = self._createExprContext(base_context,
                                                entry=processed_entry,
                                                entry_id=processed_entry[self.entry_prop])
                if not self.acl_read(context):
                    break

            readable_props = self._priv_readablePropertiesForEntry(processed_entry,
                                        base_context=base_context)

            try:
                # test each property
                for prop in processed_entry.keys():
                    if not prop in readable_props:
                        if prop in searchkeys:
                            # non-readable property was in search args
                            # so do not include this entry in
                            # found entries
                            raise Unauthorized
                        # else just remove that property
                        del processed_entry[prop]
            except Unauthorized:
                break

            # XXX should do a second pass on external search args

            returned_entries.append(processed_entry)

        return tuple(returned_entries)

    security.declarePublic('writeEntry')
    def writeEntry(self, entry_id, new_entry):
        """Write an entry in the directory"""
        return self._priv_writeEntry(entry_id, new_entry)

    def _priv_writeEntry(self, entry_id, new_entry, base_context=None):
        if base_context is None:
            base_context = self._getBaseExprContext()

        # get old entry in the directory
        old_entry = self._priv_getEntry(entry_id, base_context=base_context)

        # test acl_write
        if self.acl_write:
            context = self._createExprContext(base_context, 
                entry=old_entry, entry_id=old_entry[self.entry_prop])
            if not self.acl_write(context):
                raise Unauthorized

        # test writable properties
        writable_props = self._priv_writablePropertiesForEntry(old_entry,
                                                entry_id,
                                                base_context=base_context)
        unauthorized = [prop for prop in new_entry.keys() 
                        if not prop in writable_props]
        malformed_properties = {}
        for prop in unauthorized:
            malformed_properties[prop]='Unauthorized'

        if malformed_properties:
            return malformed_properties

        # process write
        try:
            processed_entry = self._process_write(new_entry, old_entry,
                                        entry_id=entry_id,
                                        base_context=base_context)
        except MalformedProperties, malformed_properties:
            return malformed_properties.properties
        
        # write entry
        self._writeEntry(entry_id, processed_entry)
        return None

    security.declarePublic('createEntry')
    def createEntry(self, entry):
        """
            create an entry in the directory
        """
        return self._priv_createEntry(entry)

    def _priv_createEntry(self, entry, base_context=None):
        if base_context is None:
            base_context = self._getBaseExprContext()

        if self._schema[self.entry_prop].process_write:
            context = self._createExprContext(base_context,
                                                create=1,
                                                property=entry.get(self.entry_prop,None),
                                                entry=entry)
            try:
                entry_id = self._schema[self.entry_prop].process_write(context)
            except MalformedProperties, malformed_properties:
                malformed = malformed_properties.properties
                if type(malformed) is type(''):
                    return {self.entry_prop: malformed}
                else:
                    return malformed
            entry[self.entry_prop] = entry_id
        else:
            entry_id = entry[self.entry_prop]

        old_entry = self._getEntry(entry_id)
        if old_entry:
            return {self.entry_prop: '_Already exists_'}

        # test acl_create on this new entry XXX
        if self.acl_create:
            context = self._createExprContext(base_context, entry=entry)
            if not self.acl_create(context):
                raise Unauthorized

        # get a new blank entry as an "old" entry
        old_entry = self._priv_getNewEntry(entry_id, base_context=base_context)

        try:
            # process entry
            processed_entry = self._process_write(entry, old_entry,
                                                  entry_id=entry_id)
        except MalformedProperties, malformed_properties:
            return malformed_properties.properties
        
        # create entry
        new_id = self._createEntry(entry_id, processed_entry)
        if new_id is not None:
            return new_id
        return entry_id

    security.declarePublic('getNewEntry')
    def getNewEntry(self, entry_id):
        """
        returns a new clean entry for entry_id
        """
        return self._priv_getNewEntry(entry_id)

    def _priv_getNewEntry(self, entry_id, base_context=None):
        if base_context is None:
            base_context = self._getBaseExprContext()

        return self._process_read(self._getNewEntry(entry_id),
                                        entry_id=entry_id,
                                        base_context=base_context)

    security.declarePublic('deleteEntry')
    def deleteEntry(self, entry_id):
        """
            delete an entry in the directory
        """
        return self._priv_deleteEntry(entry_id)

    def _priv_deleteEntry(self, entry_id, base_context=None):
        if base_context is None:
            base_context = self._getBaseExprContext()

        entry = self._priv_getEntry(entry_id, base_context=base_context)

        if not self._priv_can_delete(entry, base_context=base_context):
            raise Unauthorized

        if self.process_beforedelete:
            context = self._createExprContext(base_context,
                                              entry=entry,
                                              entry_id=entry[self.entry_prop])
            self.process_beforedelete(context)

        self._deleteEntry(entry_id)

    security.declarePublic('getEntry')
    def getEntry(self, entry_id):
        """
            get entry filtered by acls and processes
        """
        return self._priv_getEntry(entry_id)

    def _priv_getEntry(self, entry_id, base_context=None):
        if base_context is None:
            base_context = self._getBaseExprContext()

        entry = self._getEntry(entry_id)

        if not entry:
            return None

        prop_list = self._schema.keys()

        processed_entry = self._process_read(entry, entry_id=entry_id,
                                                base_context=base_context)

        # test read access according to acl_read
        if self.acl_read:
            context = self._createExprContext(base_context,
                                            entry=processed_entry)
            if not self.acl_read(context):
                return None

        # test read access for each property
        # remove non authorized properties
        readable_props = self._priv_readablePropertiesForEntry(processed_entry,
                                                    base_context=base_context)

        for prop in processed_entry.keys():
            if not prop in readable_props:
                del processed_entry[prop]

        return processed_entry

    security.declarePublic('reference_links')
    def reference_links(self, entries_id, portal=None, portal_url=None):
        """
        returns links references to entries_id
        returns a list of lists:
            [[{'url': URL, 'title'}, {'url': URL, 'title'}, ...] [...]]
        usually each list item in this list is just a reference to
        the entries ids but can be something hierarchical
        if entries_id is a string, it just returns a list.
        """
        if portal_url is None:
            if portal is None:
                portal = self.portal_url.getPortalObject()
            portal_url = portal.absolute_url()

        entry_prop = self.entry_prop
        display_prop = self.display_prop
        dirname = self.id

        links = []
        base_link = portal_url + '/%s?dirname=%s&entry_id=' % (
            quote_plus(self.entry_template), quote_plus(dirname))

        if type(entries_id) is type(''):
            only_one = 1
            entries_id = [entries_id]
        else:
            only_one = 0

        for entry_id in entries_id:
            entry = self.getEntry(entry_id)
            if entry is not None:
                links.append([{'url': base_link+entry[entry_prop],
                              'title': entry[display_prop]}])

        if only_one:
            return links[0]
        else:
            return links

    security.declarePublic('nomenclatureForEntries')
    def nomenclatureForEntries(self, entries=None,
                                        idprop=None, displayprop=None):
        """
        create a nomenclature with entries (default: all entries)
        """
        if entries is None:
            entries = self.searchEntry()
        if idprop is None:
            idprop = self.entry_prop
        if displayprop is None:
            displayprop = self.display_prop

        nomenclature = {}
        for entry in entries:
            nomenclature[entry[idprop]] = entry.get(displayprop, entry[idprop])

        return nomenclature

    security.declarePublic('processBinaryForm')
    def processBinaryForm(self, entry):
        """
        """
        for propid in entry.keys():
            property = self._schema.get(propid)
            if property is not None and property.type == 'binary':
                prop = entry[propid]
                del entry[propid]
                prop_change = entry.get(propid + '_change')
                if prop_change is None:
                    prop_change = 'change'
                else:
                    del entry[propid + '_change']
                if prop_change == 'change':
                    entry[propid] = str(File('temp', 'temp', prop).data)
                elif prop_change == 'delete':
                    entry[propid] = ''

    # managing the directory

    security.declarePublic('getSchemaDict')
    def getSchemaDict(self):
        schema = {}
        for propid in self._schemakeys:
            schema[propid] = self._schema[propid].__of__(aq_inner(self))
        return schema

    security.declarePublic('getSchemaKeys')
    def getSchemaKeys(self):
        return self._schemakeys

    def _getSchemaInternalKeys(self):
        return list([self._schema[key].getInternalName() 
                     for key in self._schemakeys])

    def _getInternalEntryProp(self):
        return self._schema[self.entry_prop].getInternalName()

    def _getInternalPropsForProps(self, prop_ids):
        return [self._schema[id].getInternalName() for id in prop_ids]

    def _getPropsForInternalProps(self, prop_ids):
        return [key for key in self._schemakeys
            if self._schema[key].getInternalName() in prop_ids]

    security.declarePublic('getSchemaKeysAndDict')
    def getSchemaKeysAndDict(self):
        return (self.getSchemaKeys(), self.getSchemaDict())

    security.declarePublic('getSearchableSchemaKeys')
    def getSearchableSchemaKeys(self):
        return [key for key in self._schemakeys 
                    if self._schema[key].issearchable]

    security.declarePublic('getVisibleSchemaKeys')
    def getVisibleSchemaKeys(self):
        return [key for key in self._schemakeys if self._schema[key].isvisible]

    security.declarePublic('getNomenclatures')
    def getNomenclatures(self):
        return self.nomenclatures


    #
    # private methods
    #
    def _process_read(self, entry, entry_id, base_context=None):
        """Process an entry returned by the backend directory.

           Takes a dict with alias property names as keys.
           Returns a dict with property names as keys.
        """
        if base_context is None:
            base_context = self._getBaseExprContext()

        prop_list = self._schemakeys

        # unalias entry prop names
        unaliased_entry ={}
        for prop in prop_list:
            property = self._schema[prop]
            if not property.isexternal:
                alias = property.alias or prop
                if entry.has_key(alias):
                    unaliased_entry[prop] = entry[alias]

        entry = unaliased_entry

        processed_entry = {}
        
        for prop in prop_list:
            property = self._schema[prop]
            if property.isexternal or entry.has_key(prop):
                if property.process_read:
                    property_value = entry.get(prop, None)
                    context = self._createExprContext(base_context,
                                                    entry=entry,
                                                    entry_id = entry_id,
                                                    property=property_value)
                    processed_entry[prop] = property.process_read(context)
                else:
                    processed_entry[prop] = entry[prop]

        return processed_entry
                                            

    def _process_write(self, new_entry, old_entry, entry_id, base_context=None):
        """Process new entry to be suitable to backend directory. 

        Write takes 2 dict (old and new entry) with properties name as keys.
        Returns a dict with alias properties name as keys with external
        properties removed.
        """
        if base_context is None:
            base_context = self._getBaseExprContext()

        prop_list = self._schema.keys()

        processed_entry = {}
        malformed_properties = {}
        for prop in prop_list:
            property = self._schema[prop]
            
            if new_entry.has_key(prop):
                alias = property.alias or prop
                if property.process_write:
                    context = self._createExprContext(base_context,
                                        create=0,
                                        entry=new_entry,
                                        old_entry=old_entry,
                                        entry_id=entry_id,
                                        property=new_entry.get(prop, None),
                                        old_property=old_entry.get(prop, None))
                    try:
                        processed_entry[alias] = property.process_write(context)
                    except MalformedProperties, exc:
                        malformed_properties[prop] = exc.properties

                else:
                    processed_entry[alias] = new_entry[prop]

                if processed_entry.has_key(alias):
                    processed_prop = processed_entry[alias]
                    if property.type == 'list' and processed_prop:
                        processed_entry[alias] = \
                            filter(None, processed_entry[alias])
                    if (property.isexternal or processed_prop is None):
                        del processed_entry[alias]

        if malformed_properties:
            raise MalformedProperties, malformed_properties

        return processed_entry

    def _getBaseExprContext(self):
        directories = aq_parent(aq_inner(self))
        directory = self
        portal_url = getToolByName(self, 'portal_url')
        portal = portal_url.getPortalObject()
        portal_membership = getToolByName(self, 'portal_membership')
        member = portal_membership.getAuthenticatedMember()
        roles = member.getRoles()

        data = {
            'directories': directories,
            'directory': directory,
            'portal': portal,
            'member': member,
            'nobody': None,
            'nothing': None,
            'malformed': malformed,
            'manager': 'Manager' in roles,
            'directory_manager': 'DirectoryManager' in roles
        }
        return data

    def _createExprContext(self, data, **kw):
        data.update(kw)
        return getEngine().getContext(data)

    #
    # To be implemented by subclasses
    #
    def _getEntry(self, entry_id):
        pass

    def _searchEntry(self, search_options, **kw):
        pass

    def _createEntry(self, entry_id, entry):
        pass

    def _writeEntry(self, entry_id, entry):
        pass

    def _deleteEntry(self, entry_id):
        pass

    def _getNewEntry(self, entry_id):
        pass


    #
    # ZMI
    #
    manage_editSchemaForm = DTMLFile('zmi/manage_editSchemaForm', globals())

    security.declareProtected(DirectoryEditPermission, 'manage_addSchemaItem')
    def manage_addSchemaItem(self, id, title, acl_read, acl_write,
                            process_read, process_write, REQUEST=None):
        """Add a new schema item to a directory."""
        if not id in self._schema.keys():
            item = DirectorySchemaProperty(id, title=title,
                        acl_read=acl_read, acl_write=acl_write,
                        process_read=process_read, process_write=process_write)
            item.directory = aq_base(self)
            self._schema[id] = item
            self._schemakeys = self._schemakeys + (id,)

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(self.absolute_url()
                + '/manage_editSchemaForm?manage_tabs_message=Added.')

    security.declareProtected(DirectoryEditPermission, 'manage_deleteSchemaItems')
    def manage_deleteSchemaItems(self, select_ids=[], REQUEST=None, **kw):
        """Delete schema items."""
        for id in select_ids:
            del self._schema[id]

        self._schemakeys = tuple([id for id in self._schemakeys
                                     if id not in select_ids])
        
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(self.absolute_url() 
                + '/manage_editSchemaForm?manage_tabs_message=Deleted.')

    manage_organizeSchemaItemsForm = DTMLFile('zmi/manage_organizeSchemaItemsForm', globals())

    security.declareProtected(DirectoryEditPermission, 'manage_organizeSchemaItems')
    def manage_organizeSchemaItems(self, id, up, REQUEST=None):
        """Move up or down one schema property."""
        up = int(up)
        keys = list(self._schemakeys)
        lenkeys = len(keys)
        index = keys.index(id)
        newindex = index + (up and -1 or 1)
        if newindex>=0 and newindex<lenkeys:
            oldid = keys[newindex]
            keys[newindex] = id
            keys[index] = oldid
        self._schemakeys = tuple(keys)

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(self.absolute_url() 
                + '/manage_organizeSchemaItemsForm?manage_tabs_message=Moved %s' % (up and 'Up' or 'Down'))

    security.declareProtected(DirectoryEditPermission, 'manage_editSchemaItems')
    def manage_editSchemaItems(self, ids=[], REQUEST=None, **kw):
        """Edit schema items according to the form."""
        if REQUEST is not None:
            kw.update(REQUEST.form)

        schema = self._schema
        keys = self._schemakeys
        newkeys = []
        for index in range(len(keys)):
            id = keys[index]
            item = schema[id]
            item.directory = aq_base(self) # XXX to be reviewed
            new_id = ids[index]
            if new_id != id:
                if new_id in keys:
                    # this id name already exists
                    raise AttributeError, "can't rename %s to %s" % (id, new_id)
                item.id = new_id
                del schema[id]
                schema[new_id] = item
                newkeys.append(new_id)
            else:
                newkeys.append(id)
            attr_change = {}
            for attr in ('acl_read', 'acl_write', 'process_read', 
              'process_write', 'nomenclature'):
                name = "%s__%s" % (id, attr)
                new = kw.get(name, None)
                if new is None:
                    continue
                new = new.strip()
                expr = getattr(item, attr)
                if expr is not None:
                    expr_text = expr.text
                else:
                    expr_text = ''
                if new != expr_text:
                    attr_change[attr] = new
            for attr in ('isexternal', 'isvisible', 'issearchable', 
              'isdirectory_ref'):
                name = "%s__%s" % (id, attr)
                new = kw.get(name, None)
                if new is None:
                    new = 0
                attr_change[attr] = new
            for attr in ('alias', 'title', 'type', 'directory_ref'):
                name = "%s__%s" % (id, attr)
                new = kw.get(name, None)
                if new is None:
                    continue
                if getattr(item, attr) != new:
                    attr_change[attr] = new

            item._edit(**attr_change)

        self._schemakeys = tuple(newkeys)

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(self.absolute_url() 
                + '/manage_editSchemaForm?manage_tabs_message=Changed.')


    manage_propertiesForm = DTMLFile('zmi/manage_propertiesForm', globals())

    security.declareProtected(DirectoryEditPermission, 'manage_properties')
    def manage_properties(self, title=None, description=None,
                          entry_prop=None, display_prop=None, acl_read=None,
                          acl_write=None, acl_create=None, acl_delete=None,
                          process_aftercreate=None, process_beforedelete=None,
                          search_template=None, results_template=None,
                          entry_template=None, edit_template=None,
                          REQUEST=None):
        """Change directory's properties."""
        if title is not None:
            self.title = title
        if entry_prop is not None:
            self.entry_prop = entry_prop
        if display_prop is not None:
            self.display_prop = display_prop
        if description is not None:
            self.description = description
        if search_template is not None:
            self.search_template = search_template
        if results_template is not None:
            self.results_template = results_template
        if entry_template is not None:
            self.entry_template = entry_template
        if edit_template is not None:
            self.edit_template = edit_template
        self._changeExpression(acl_read=acl_read,
                               acl_write=acl_write,
                               acl_create=acl_create,
                               acl_delete=acl_delete,
                               process_aftercreate=process_aftercreate,
                               process_beforedelete=process_beforedelete)

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(
                'manage_propertiesForm?manage_tabs_message=Changed.')

    def _changeExpression(self, **kw):
        for expr in kw.keys():
            value = kw[expr]
            if value is not None:
                if value:
                    setattr(self, expr, Expression(value))
                else:
                    setattr(self, expr, None)

InitializeClass(BaseDirectory)
