# (c) 2002 Nuxeo SARL <http://nuxeo.com>
# (c) 2002 Florent Guillaume <mailto:fg@nuxeo.com>
# (c) 2002 Prfecture du Bas-Rhin, France
# See license info at the end of this file.
# $Id: Workgroup.py,v 1.17 2003/04/11 13:24:32 guillaum Exp $

"""
  Workgroup
"""

import urllib

from AccessControl import ClassSecurityInfo, getSecurityManager
from Globals import InitializeClass, DTMLFile
from Acquisition import aq_base

from Products.CMFCore.CMFCorePermissions import \
     setDefaultRoles, View, ManageProperties
from Products.CMFCore.utils import getToolByName
#from Products.CMFDefault.SkinnedFolder import SkinnedFolder

from Products.PortalContentFolder.PortalContentFolder import PortalContentFolder



AddWorkgroups = 'Add Workgroups'
ManageWorkgroups = 'Manage Workgroups'
setDefaultRoles(AddWorkgroups, ('Manager',))
setDefaultRoles(ManageWorkgroups, ('Manager',))


WorkgroupManager = 'WorkgroupManager'
WorkgroupMember = 'WorkgroupMember'
WorkgroupVisitor = 'WorkgroupVisitor'
WorkgroupManagerRoles = (WorkgroupManager, WorkgroupMember, WorkgroupVisitor,)
WorkgroupMemberRoles = (WorkgroupMember, WorkgroupVisitor,)
WorkgroupVisitorRoles = (WorkgroupVisitor,)



factory_type_information = (
    {'id': 'Workgroup',
     'title': '_portal_type_Workgroup',
     'icon': 'workgroup_icon.gif',
     'product': 'NuxWorkgroup',
     'meta_type': 'Workgroup',
     'factory': 'addWorkgroup',
     'immediate_view': 'workgroup_edit_form',
     'filter_content_types': 1,
     'allowed_content_types': (
                               'Workgroup',
                               ),
     'actions': ({'id': 'view',
                  'name': '_action_view_',
                  'action': 'workgroup_contents',
                  'permissions': (View,),
                  'category': 'object',
                  },
                 {'id': 'edit',
                  'name': '_action_modify_',
                  'action': 'workgroup_edit_form',
                  'permissions': (ManageProperties,),
                  'category': 'object'
                  },
                 {'id': 'localroles',
                  'name': '_action_access_rights_',
                  'action': 'workgroup_localrole_form',
                  'permissions': (ManageWorkgroups,),
                  'category': 'object'
                  },
                 {'id': 'create',
                  'name': '_action_create_',
                  'action': 'workgroup_create_form',
                  'visible': 0,
                  'permissions': ()},
                 {'id': 'isfunctionalobject',
                  'name': 'isfunctionalobject',
                  'action': 'isfunctionalobject',
                  'visible': 0,
                  'permissions': ()},
                 ),
     },
    )


class Workgroup(PortalContentFolder):
    """
    A Workgroup is a PortalContentFolder whose local roles can be
    easily managed.
    """
    meta_type = 'Workgroup'

    _properties = ({'id':'title', 'type':'string'},
                   {'id':'description', 'type':'text'},
                   )

    isInWorkgroup = 1 # XXX used where ?
    isPortalContent = 0
    isDocumentContainer = 1 # to distinguish folder view method from hierarchies

    security = ClassSecurityInfo()
    security.declareObjectProtected(View)

    def __init__(self, id,
                 title='', description=''):
        PortalContentFolder.__init__(self, id, title)
        self.description = description

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        """Adds self to the workflow and catalog."""
        PortalContentFolder.manage_afterAdd(self, item, container)
        # Fixup local roles
        user = getSecurityManager().getUser()
        if user is not None:
            name = user.getUserName()
            if name != 'Anonymous User':
                self.addManagers([name])

    _basetitle = '_private_area_of_%s_'
    _computedtitle = 0

    security.declareProtected(View, 'isTitleDynamic')
    def isTitleDynamic(self):
        return self._computedtitle

    security.declareProtected(View, 'title_or_id')
    def title_or_id(self):
        """Returns title or id as usual, except for private areas"""
        if not self._computedtitle:
            return PortalContentFolder.title_or_id(self)
        else:
            dirtool = getToolByName(self, 'portal_metadirectories')
            mcat = getToolByName(self, 'portal_messages')
            members = dirtool.members
            title = None
            entry = members.getEntry(self.id)
            if entry is not None:
                title = entry.get(members.display_prop)
            if title is None:
                return PortalContentFolder.title_or_id(self)
            if self._basetitle.find('%s') > -1:
                title = mcat(self._basetitle) % (title, )
            else:
                title = mcat(self._basetitle)
            return title

    security.declareProtected(ManageProperties, 'edit')
    def edit(self,
             title=None,
             description=None,
             **kw):
        """Edits the workgroup title and description."""
        if title is not None:
            self.title = title
        if description is not None:
            self.description = description
        self.reindexObject()

    security.declareProtected(ManageProperties, 'setTitle')
    def setTitle(self, title):
        """Dublin Core title"""
        self.edit(title=title)
        self.reindexObject()

    security.declareProtected(View, 'thisWorkgroup')
    def thisWorkgroup(self):
        """Returns the current workgroup."""
        return self

    #
    # Local roles management
    #

    # adding users

    security.declarePrivate('_addUserRoles')
    def _addUserRoles(self, ids, roles, REQUEST=None, msg=''):
        """Adds users."""
        for userid in ids:
            self.manage_addLocalRoles(userid, roles)
        self.reindexObjectSecurity()
        if REQUEST is not None:
            psm = '?portal_status_message=%s' % urllib.quote(msg)
            REQUEST.RESPONSE.redirect(self.absolute_url()+'/workgroup_localrole_form'+psm)

    security.declareProtected(ManageWorkgroups, 'addManagers')
    def addManagers(self, ids, REQUEST=None):
        """Adds owners."""
        self._addUserRoles(ids, WorkgroupManagerRoles, REQUEST, msg='Propritaires ajouts.')

    security.declareProtected(ManageWorkgroups, 'addMembers')
    def addMembers(self, ids, REQUEST=None):
        """Adds members."""
        self._addUserRoles(ids, WorkgroupMemberRoles, REQUEST, msg='Members ajouts.')

    security.declareProtected(ManageWorkgroups, 'addVisitors')
    def addVisitors(self, ids, REQUEST=None):
        """Adds visitors."""
        self._addUserRoles(ids, WorkgroupVisitorRoles, REQUEST, msg='Visiteurs ajouts.')

    # adding groups

    security.declarePrivate('_addGroupRoles')
    def _addGroupRoles(self, groupids, roles, REQUEST=None, msg=''):
        """Adds groups."""
        for groupid in groupids:
            self.manage_addLocalGroupRoles(groupid, roles)
        self.reindexObjectSecurity()
        if REQUEST is not None:
            psm = '?portal_status_message=%s' % urllib.quote(msg)
            REQUEST.RESPONSE.redirect(self.absolute_url()+'/workgroup_localrole_form'+psm)

    security.declareProtected(ManageWorkgroups, 'addGroupManagers')
    def addGroupManagers(self, groupids, REQUEST=None):
        """Adds group owners."""
        self._addGroupRoles(groupids, WorkgroupManagerRoles, REQUEST, msg="Propritaires ajouts")

    security.declareProtected(ManageWorkgroups, 'addGroupMembers')
    def addGroupMembers(self, groupids, REQUEST=None):
        """Adds group members."""
        self._addGroupRoles(groupids, WorkgroupMemberRoles, REQUEST, msg="Members ajouts")

    security.declareProtected(ManageWorkgroups, 'addGroupVisitors')
    def addGroupVisitors(self, groupids, REQUEST=None):
        """Adds group visitors."""
        self._addGroupRoles(groupids, WorkgroupVisitorRoles, REQUEST, msg="Visiteurs ajouts")

    # removing users

    security.declareProtected(ManageWorkgroups, 'delUsers')
    def delUsers(self, ids, REQUEST=None):
        """ Dels users. """
        for userid in ids:
            lroles = self.get_local_roles_for_userid(userid)
            newroles = filter(lambda r: r not in WorkgroupManagerRoles, lroles)
            if newroles:
                self.manage_setLocalRoles(userid, list(newroles))
            else:
                self.manage_delLocalRoles((userid,))
        self.reindexObjectSecurity()
        if REQUEST is not None:
            psm = '?portal_status_message=Utilisateurs+supprims.'
            REQUEST.RESPONSE.redirect(self.absolute_url()+'/workgroup_localrole_form'+psm)

    # removing groups

    security.declareProtected(ManageWorkgroups, 'delGroups')
    def delGroups(self, groupids, REQUEST=None):
        """ Dels groups. """
        for groupid in groupids:
            lroles = self.get_local_roles_for_groupid(groupid)
            newroles = filter(lambda r: r not in WorkgroupManagerRoles, lroles)
            if newroles:
                self.manage_setLocalGroupRoles(groupid, list(newroles))
            else:
                self.manage_delLocalGroupRoles((groupid,))
        self.reindexObjectSecurity()
        if REQUEST is not None:
            psm = '?portal_status_message=Groupes+supprims.'
            REQUEST.RESPONSE.redirect(self.absolute_url()+'/workgroup_localrole_form'+psm)


InitializeClass(Workgroup)


def addWorkgroup(dispatcher, id,
                 title='',
                 description='',
                 REQUEST=None):
    """Adds a Workgroup."""
    ob = Workgroup(id, title, description)
    container = dispatcher.Destination()
    container._setObject(id, ob)
    #ob = container._getOb(id)
    if REQUEST is not None:
        url = dispatcher.DestinationURL()
        REQUEST.RESPONSE.redirect('%s/manage_main' % url)


# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
