# Copyright (c) 2002 Nuxeo SARL <http://nuxeo.com>
# Copyright (c) 2002 Prfecture du Bas-Rhin, France
# Copyright (c) 2002 Adelux <http://adequat.c2a.fr>
# Author: Florent Guillaume <mailto:fg@nuxeo.com>
# See license info at the end of this file.
# $Id: MiscTools.py,v 1.3 2003/06/12 08:43:03 fermigier Exp $

import string

from Acquisition import aq_base, aq_inner, aq_parent
from AccessControl.Role import gather_permissions
from AccessControl.Permission import Permission

from OFS.CopySupport import CopyError, eNotSupported


def id_in_folder(folder,id,start_with=''):
    """ """
    prefix=id
    suffix=start_with
    id='%s%s'%(prefix,suffix)
    if type(suffix) is not type(0):
        suffix=0
    while id in folder.objectIds():
        suffix=suffix+1
        id='%s%s'%(prefix,suffix)
    return id

def legal_id(id):
    """ id manipulations do here: """
    return string.translate(id,string.maketrans(' ','_eeaceei'))

def cut_and_paste(to,ob,paste_as=''):
    """ """
    orig = aq_parent(aq_inner(ob))

    ob._notifyOfCopyTo(to, op=1)

    if not ob.cb_isCopyable():
        raise CopyError, eNotSupported % ob.getId()

    id = ob.getId()

    # try to make ownership explicit
    ob.manage_changeOwnershipType(explicit=1)

    orig._delObject(id)
    ob = aq_base(ob)
    new_id = paste_as or id
    new_id = to._get_id(new_id)
    ob._setId(new_id)

    to._setObject(new_id, ob, set_owner=0)

    ob = to._getOb(new_id)
    ob.manage_changeOwnershipType(explicit=0)
    #return getattr(to,new_id)

def copy_and_paste(to,ob,paste_as=''):
    """ """
    orig = aq_parent(aq_inner(ob))

    ob._notifyOfCopyTo(to, op=0)

    if not ob.cb_isCopyable():
        raise CopyError, eNotSupported % ob.getId()

    ob=ob._getCopy(to)
    ob.manage_afterClone(ob)

    new_id=paste_as or ob.getId()
    new_id=to._get_id(new_id)

    ob._setId(new_id)
    to._setObject(new_id, ob)

    #return getattr(to,new_id)

def rename(ob, new_id):
    """Rename object ob to new_id"""
    container = aq_parent(aq_inner(ob))
    ob._notifyOfCopyTo(container, op=1)

    old_id = ob.getId()
    container._delObject(old_id)

    ob = aq_base(ob)
    ob._setId(new_id)
    container._setObject(new_id, ob, set_owner=0)

def get_roles_for_permission(obj, permission):
    """ """
    valid_roles = self.valid_roles()
    for p in self.ac_inherited_permissions(1):
        name, value = p[:2]
        if name == permission:
            p = Permission(name,value,self)
            return p.getRoles()

def getPortal(ob):
    """ """
    while ob is not None and not getattr(ob, '_isPortalRoot', 0):
        ob = aq_parent(aq_inner(ob))
    return ob


def ac_inherited_permissions(ob, all=0):
    # Get all permissions not defined in ourself that are inherited
    # This will be a sequence of tuples with a name as the first item and
    # an empty tuple as the second.
    d = {}
    perms = getattr(ob, '__ac_permissions__', ())
    for p in perms: d[p[0]] = None
    r = gather_permissions(ob.__class__, [], d)
    if all:
        if hasattr(ob, '_subobject_permissions'):
            for p in ob._subobject_permissions():
                pname=p[0]
                if not d.has_key(pname):
                    d[pname]=1
                    r.append(p)
        r = list(perms) + r
    return r

def modifyRolesForPermission(ob, pname, roles):
    '''
    Modifies multiple role to permission mappings.  roles is a list to
    acquire, a tuple to not acquire.
    '''
    # This mimics what AccessControl/Role.py does.
    data = ()
    for perm in ac_inherited_permissions(ob, 1):
        name, value = perm[:2]
        if name == pname:
            data = value
            break
    p = Permission(pname, data, ob)
    if p.getRoles() != roles:
        p.setRoles(roles)
        return 1
    return 0


# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
